/***************************************************************************

	FILE NAME: DRVR6210.C

	FILE DESCRIPTION: Driver Functions for DM6210

	This file is a library of functions that make up the "Driver" for
	the DM6210.  It must be included with any of the DM6210 example
	programs.

	PROJECT NAME: DM6210 DOS Driver

	DRIVER VERSION: 1.1

	COMPILER: Borland C++ 3.1

	TARGET: Real-Mode DOS

	Copyright 2003 RTD Embedded Technologies

***************************************************************************/


#include <conio.h>
#include <dos.h>
#include <stdio.h>
#include <string.h>

#include "..\DRIVER\DRVR6210.h"

// Board Registers
#define r_AD_6210								0		// AD Data (Read)
#define r_START_CONVERSION_6210			0		// Start Conversion (Write)
#define r_BOARD_ID_LSB_6210				2		// Board ID LSB (Read)
#define r_BOARD_ID_MSB_6210				3		// Board ID MSB (Read)
#define r_CHANNEL_6210						4		// Channel Register (Read/Write)
#define r_IRQ_6210							5		// IRQ Register (Read/Write)
#define r_STATUS_6210						6		// Status Register (Read)
#define r_CLEAR_IRQ_6210					6		// Clear IRQ (Write)
#define r_TIMER_CLCK0_6210					8		// Timer/Counter 0 (Read/Write)
#define r_TIMER_CLCK1_6210					9		// Timer/Counter 1 (Read/Write)
#define r_TIMER_CLCK2_6210					10		// Timer/Counter 2 (Read/Write)
#define r_TIMER_CTRL_6210					11		// Timer/Counter Control Word (Write)
#define r_DIO_PORT_0_6210					12		// Digital I/O Port 0 Register (Read/Write)
#define r_DIO_PORT_1_6210					13	   // Digital I/O Port 1 Register (Read/Write)
#define r_DIO_STROBE_SEL_6210				14 	// Digital Input Strobe Select Register (Read/Write)
#define r_DIO_CONTROL_6210					15 	// Digital I/O Control Register (Read/Write)





char* TitleString6210(void)
{
	return "DM6210 sample program.";
} //TitleString6210




//*************************************************************************
//	InitBoard6210
//
//	This Routine Should allways be called first.
//	This clears the board and varibles the driver uses.
//
//	Parameters:
//		BaseAddress:		BaseAddress of board.
//*************************************************************************
void InitBoard6210(uint16 BaseAddress)
{
	ClearIRQ6210(BaseAddress);
} // InitBoard6210




//************************ READ A/D DATA **********************************
//	Read at BA + 0

//*************************************************************************
//	ReadADData6210
//
//	This Routine Reads the A/D Data.
//
//	Parameters:
//		BaseAddress:		BaseAddress of board.
//
//		Returns: 			Unsigned 12-bit A/D data.
//*************************************************************************
uint16  ReadADData6210(uint16 BaseAddress)
{
	return((uint16) inpw(BaseAddress + r_AD_6210) >> 4);
} // ReadADData6210




//************************ START CONVERSION *******************************
//	Write at BA + 0

//*************************************************************************
//	StartConversion6210
//
//	This routine is used to start A/D conversions.
//
//	Parameters:
//		BaseAddress:		BaseAddress of board.
//*************************************************************************
void StartConversion6210(uint16 BaseAddress)
{
	outp(BaseAddress + r_START_CONVERSION_6210, 0);
} // StartConversion6210




//************************ RESERVED ************************
//	Read/Write at BA + 1



//************************ BOARD ID ************************
//	Read at BA + 2

//*************************************************************************
//	ReadBoardID6210
//
//	This routine is used to read the board ID register.
//
//	Parameters:
//		BaseAddress:	BaseAddress of board.
//
//		Returns the 16-bit ID.
//*************************************************************************
uint16 ReadBoardID6210(uint16 BaseAddress)
{
	uchar8 MSB, LSB;

	LSB = inp(BaseAddress + r_BOARD_ID_LSB_6210);
	MSB = inp(BaseAddress + r_BOARD_ID_MSB_6210);
	return((256 * MSB) + LSB);
} // ReadBoardID6210




//************************ RESERVED ************************
//	Write at BA + 2




//************************ LOAD CHANNEL ******************************
//	Read/Write at BA + 4

//*************************************************************************
//	SetChannel6210
//
//	This routine loads the channel latch.
//
//	Parameters:
//		BaseAddress:	BaseAddress of board.
//
//			Channel:		0 - 15
//
//*************************************************************************
void SetChannel6210(uint16 BaseAddress, uint16 Channel)
{
	outp(BaseAddress + r_CHANNEL_6210, Channel);  // send to board
} // SetChannel6210




//************************ IRQ REGISTER ***********************************
//	Read/Write at BA + 5

//*************************************************************************
//	SetIRQ6210
//
//	This routine sets the source and channel for the interrupt.
//
//	Parameters:
//		BaseAddress:		BaseAddress of board.
//
//		Source:		0 = A/D End-of-Convert
//  					1 = Timer/Counter Out 0
//						2 = Timer/Counter Out 1
//						3 = Timer/Counter Out 2
//						4 = External Clock 2
//						5 = P0 Latch Status
//						6 = P1 Latch Status
//						7 = Reserved
//
// 	Channel:		0 = disabled
//						1 = IRQ 3
//						2 = IRQ 5
//						3 = IRQ 9
//						4 = IRQ 10
//						5 = IRQ 11
//						6 = IRQ 12
//						7 = IRQ 15
//*************************************************************************
void SetIRQ6210(uint16 BaseAddress, uchar8 Source, uchar8 Channel)
{
	switch(Channel) {
		case 3:Channel = 1;
		break;
		case 5: Channel = 2;
		break;
		case 9: Channel = 3;
		break;
		case 10: Channel = 4;
		break;
		case 11: Channel = 5;
		break;
		case 12: Channel = 6;
		break;
		case 15: Channel = 7;
		break;
		default: Channel = 0;
		break;
	}

	outp(BaseAddress + r_IRQ_6210, Source | (Channel << 3));
} // SetIRQ6210




//************************ QUERY ROUTINES *********************************
//	Read at BA + 6

//*************************************************************************
//	ReadStatus6210
//
//	This routine returns the status from the board.
//
//	Parameters:
//		BaseAddress:		BaseAddress of board.
//
//		Returns:				8 bit status word.
//*************************************************************************
uchar8 ReadStatus6210(uint16 BaseAddress)
{
 return(inp(BaseAddress + r_STATUS_6210));
} // ReadStatus6210




//*************************************************************************
//	IsADConverting6210
//
//	This routine checks to see if the AD is converting.
//
//	Parameters:
//		BaseAddress:		BaseAddress of board.
//
//		Returns:				1 if AD is converting
//								0 if AD is not converting
//*************************************************************************
uchar8 IsADConverting6210(uint16 BaseAddress)
{
	uchar8 Status;

	Status = inp(BaseAddress + r_STATUS_6210);
	Status = Status & 1;
	return(!Status);
} // IsADConverting6210




//*************************************************************************
//	IsIRQ6210
//
//	This routine checks the IRQ status bit.
//
//	Parameters:
//		BaseAddress:		BaseAddress of board.
//
//		Returns:				1 if IRQ has been generated
//								0 if no IRQ
//*************************************************************************
uchar8 IsIRQ6210(uint16 BaseAddress)
{
	uchar8 Status;

	Status = inp(BaseAddress + r_STATUS_6210);
	Status = (Status >> 1) & 1;
	return(Status);
} // IsIRQ6210




//*************************************************************************
//	IsP0Strobe6210
//
//	This routine checks to see if the P0 input strobe has occured.
//
//	Parameters:
//		BaseAddress:		BaseAddress of board.
//
//		Returns:				1 if strobe has occured
//								0 if no strobe
//*************************************************************************
uchar8 IsP0Strobe6210(uint16 BaseAddress)
{
	uchar8 Status;

	Status = inp(BaseAddress + r_STATUS_6210);
	Status = (Status >> 2) & 1;
	return(Status);
} // IsP0Strobe6210




//*************************************************************************
//	IsP1Strobe6210
//
//	This routine checks to see if the P1 input strobe has occured.
//
//	Parameters:
//		BaseAddress:		BaseAddress of board.
//
//		Returns:				1 if strobe has occured
//								0 if no strobe
//*************************************************************************
uchar8 IsP1Strobe6210(uint16 BaseAddress)
{
	uchar8 Status;

	Status = inp(BaseAddress + r_STATUS_6210);
	Status = (Status >> 3) & 1;
	return(Status);
} // IsP1Strobe6210




//************************ CLEAR ROUTINES *********************************
//	Write at BA + 6

//*************************************************************************
//	ClearIRQ6210
//
//	This routine is used to clear the IRQ circuitry and status bit.
//
//	Parameters:
//		BaseAddress:		BaseAddress of board.
//*************************************************************************
void ClearIRQ6210(uint16 BaseAddress)
{
	outp(BaseAddress + r_CLEAR_IRQ_6210, 0);
} // ClearIRQ6210




//************************ RESERVED *******************************
//	Read/Write at BA + 7




//************************ TIMER COUNTER ROUTINES **************************

//*************************************************************************
//	ClockMode6210
//
//	This routine is used to set the mode of a designated counter
//	in the 8254 programmable interval timer (PIT).
//
//	Parameters:
//		BaseAddress:	BaseAddress of board.
//
//		Timer:		0,1,2
//
//		Mode:			0,1,2,3,4,5
//*************************************************************************
void ClockMode6210(uint16 BaseAddress, uchar8 Timer, uchar8 Mode)
{
	uchar8 StatusByte;

	StatusByte = (Timer << 6) + (Mode << 1) + 0x30;
	outp(BaseAddress + r_TIMER_CTRL_6210, StatusByte);
} // ClockMode6210




//*************************************************************************
//	ClockDivisor6210
//
//	This routine is used to set the divisor of a designated
//	counter on the 8254 programmable interval timer (PIT).  This procedure
//	assumes that the counter has already been set to receive the least
//	significant uchar8 (LSB) of the divisor followed by the most significant
//	uchar8 (MSB).
//
//	Parameters:
//		BaseAddress:	BaseAddress of board.
//
//		Timer:		0,1,2
//
//		Divisor:		0 - 65535
//*************************************************************************
void ClockDivisor6210(uint16 BaseAddress, uchar8 Timer, uint16 Divisor)
{
	uchar8 MSB, LSB;

	LSB = Divisor & 0xff;
	MSB = (Divisor & 0xff00) >> 8;
	outp(BaseAddress + r_TIMER_CLCK0_6210 + Timer, LSB);
	outp(BaseAddress + r_TIMER_CLCK0_6210 + Timer, MSB);
} // ClockDivisor6210




//*************************************************************************
//	SetUserClock6210
//
//	This routine sets the user timer counters.
//
//	Parameters:
//		BaseAddress:	BaseAddress of board.
//
//		Timer:			0, 1, 2
//
//		InputRate:		Input clock to selected timer.
//
//		OutputRate:		Desired output rate from selected timer.
//
// Returns:
//		The actual clock frequency that is programmed.
//*************************************************************************
float32 SetUserClock6210(uint16 BaseAddress, uchar8 Timer, float32 InputRate, float32 OutputRate)
{
	ClockMode6210(BaseAddress, Timer, 2);
	ClockDivisor6210(BaseAddress, Timer, (InputRate/OutputRate));

	return(InputRate / (int)(InputRate/OutputRate));

} // SetUserClock6210




//*************************************************************************
//	ReadTimerCounter6210
//
//	This routine is used to read the contents of the selected timer/counter.
//
//	Parameters:
//		BaseAddress:	BaseAddress of board.
//
//		Timer:			0,1,2
//
//		Returns the 16-bit count value.
//*************************************************************************
uint16 ReadTimerCounter6210(uint16 BaseAddress, uchar8 Timer)
{
	uchar8 MSB, LSB;

	outp(BaseAddress + r_TIMER_CTRL_6210, (Timer << 6));		//send latch command
	LSB = inp(BaseAddress + r_TIMER_CLCK0_6210 + Timer);
	MSB = inp(BaseAddress + r_TIMER_CLCK0_6210 + Timer);
	return((256 * MSB) + LSB);
} // ReadTimerCounter6210




//*************************************************************************
//	DoneTimer6210
//
//	Initialize the timers for high speed to ensure the immediate load.
//
//	Parameters:
//		BaseAddress:	BaseAddress of board.
//*************************************************************************
void DoneTimer6210(uint16 BaseAddress)
{
	ClockMode6210(BaseAddress, 0, 2);
	ClockMode6210(BaseAddress, 1, 2);
	ClockDivisor6210(BaseAddress, 0, 2);
	ClockDivisor6210(BaseAddress, 1, 2);
} // DoneTimer6210




//************************ DIGITAL I/O ROUTINES **************************

//*************************************************************************
//	DIOSetPort0Direction6210
//
//	This routine sets the Port 0 low and Port 0 high direction.
//
//	Parameters:
//		BaseAddress:		BaseAddress of board.
//
//		LDirection:			0 = In
//								1 = Out
//
//		HDirection:			0 = In
//								1 = Out
//*************************************************************************
void DIOSetPort0Direction6210(uint16 BaseAddress, uchar8 LDirection, uchar8 HDirection)
{
	uchar8 Mode;

	Mode = inp(BaseAddress  + r_DIO_CONTROL_6210);
	Mode &= 0xFC;
	Mode = Mode | LDirection | (HDirection << 1);
	outp(BaseAddress + r_DIO_CONTROL_6210, Mode);			// send to board
} //DIOSetPort0Direction6210




//*************************************************************************
//	DIOSetPort1Direction6210
//
//	This routine sets the Port 1 low and Port 0 high direction.
//
//	Parameters:
//		BaseAddress:		BaseAddress of board.
//
//		LDirection:			0 = In
//								1 = Out
//
//		HDirection:			0 = In
//								1 = Out
//*************************************************************************
void DIOSetPort1Direction6210(uint16 BaseAddress, uchar8 LDirection, uchar8 HDirection)
{
	uchar8 Mode;

	Mode = inp(BaseAddress  + r_DIO_CONTROL_6210);
	Mode &= 0xF3;
	Mode = Mode | (LDirection << 2) | (HDirection << 3);
	outp(BaseAddress + r_DIO_CONTROL_6210, Mode);			// send to board
} //DIOSetPort1Direction6210




//*************************************************************************
//	DIOLatchEnable6210
//
//	This routine is used to enable the latched input for Port 0 and Port 1.
//
//	Parameters:
//		BaseAddress:		BaseAddress of board.
//
//		EnableP0:			0 = Disabled
//								1 = Enabled
//
//		EnableP1:			0 = Disabled
//								1 = Enabled
//*************************************************************************
void DIOLatchEnable6210(uint16 BaseAddress, uchar8 EnableP0, uchar8 EnableP1)
{
	uchar8 Mode;

	Mode = inp(BaseAddress  + r_DIO_CONTROL_6210);
	Mode &= 0xCF;
	Mode = Mode | (EnableP0 << 4) | (EnableP1 << 5);
	outp(BaseAddress + r_DIO_CONTROL_6210, Mode);			// send to board
} //DIOLatchEnable6210




//*************************************************************************
//	DIOLatchStrobePolarity6210
//
//	This routine is used to set the latched input strobe polarity for Port 0 and Port 1.
//
//	Parameters:
//		BaseAddress:		BaseAddress of board.
//
//		PolarityP0:			0 = Positive edge
//								1 = Negative edge
//
//		PolarityP1:			0 = Positive edge
//								1 = Negative edge
//*************************************************************************
void DIOLatchStrobePolarity6210(uint16 BaseAddress, uchar8 PolarityP0, uchar8 PolarityP1)
{
	uchar8 Mode;

	Mode = inp(BaseAddress  + r_DIO_CONTROL_6210);
	Mode &= 0x3F;
	Mode = Mode | (PolarityP0 << 6) | (PolarityP1 << 5);
	outp(BaseAddress + r_DIO_CONTROL_6210, Mode);			// send to board
} //DIOLatchStrobePolarity6210




//*************************************************************************
//	DIOSelectStrobe6210
//
//	This routine selects the digital input strobe source.
//
//	Parameters:
//		BaseAddress:		BaseAddress of board.
//
//		Strobe:				0 = External Clock 0
//								1 = External Clock 1
//								2 = External Clock 2
//								3 = Reserved
//*************************************************************************
void DIOSelectStrobe6210(uint16 BaseAddress, uchar8 Strobe)
{
	outp(BaseAddress + r_DIO_STROBE_SEL_6210, Strobe);			// send to board
} // DIOSelectStrobe6210




//*************************************************************************
//	DIORead6210
//
//	This routine reads the DIO chip's port.
//
//	Parameters:
//		BaseAddress:		BaseAddress of board.
//
//		Port:					0, 1
//*************************************************************************
uchar8 DIORead6210(uint16 BaseAddress, uchar8 Port)
{
	return (inp(BaseAddress + r_DIO_PORT_0_6210 + Port));
} // DIORead6210




//*************************************************************************
//	DIOWrite6210
//
//	This routine writes the DIO chip's port.
//
//	Parameters:
//		BaseAddress:		BaseAddress of board.
//
//		Port:					0, 1
//
//		Data:					0 - 255
//*************************************************************************
void DIOWrite6210(uint16 BaseAddress, uchar8 Port, uchar8 Data)
{
	outp(BaseAddress + r_DIO_PORT_0_6210 + Port, Data);
} // DIOWrite6210




