/***************************************************************************

	FILE NAME: INTRPTS.C

	FILE DESCRIPTION: Interrupts Example Program for DM6210

	Sample program that demonstrates how to use the timer as an interrupt
	source and how to write an Interrupt Service Routine (ISR).

	This program sets up the User Timer to generate interrupts at a selected
	rate. An interrupt service routine (ISR) is provided that simply samples
	from one channel on the A/D board each time the interrupt is activated.
	The value is displayed on the screen as a foreground process. Although
	the ISR in this program is trivial, all the ingredients for powerful
	interrupt driven data acquisition are included.

	PROJECT NAME: INTRPTS (Part of DM6210 DOS Driver)

	DRIVER VERSION: 1.1

	COMPILER: Borland C++ 3.1

	TARGET: Real-Mode DOS

	Copyright 2003 RTD Embedded Technologies

***************************************************************************/

#include <conio.h>
#include <dos.h>

#include "..\driver\drvr6210.h"
#include "..\driver\pcutil.h"

/***************************************************************************
  Defines

  Change these constants to alter the program parameters.
****************************************************************************/

#define BOARD0	 0				            // Board 0 array pointer.
#define CHANNEL 0                      // A/D channel.
#define RATE   20                      // Clock rate in 'Hz'.
#define ADSLOPE (4096.0/10.0)          // Number Bits divided by AD Range.
#define IRQ_CHANNEL 11                 // IRQ Channel.

/****************************************************************************
	Global variables
****************************************************************************/

unsigned char BOARD;
unsigned int BASE_ADDRESS[32];


volatile int Value;                    // Global variable to store the data
													// acquired by the Interrupt Service
													// Routine (ISR). The foreground
													// program and the ISR communicate
													// trought this variable.


/****************************************************************************
  NewISR()

  The NewISR function is called whenever an interrupt is generated on the
  specified IRQ. It simply gets a single value from the A-D board.

  Do not try to access any functions that use DOS calls (disk access, screen
  writes, read keyboard, etc) and do not use floating point operations in
  this function. DOS and the Floating Point Emulator are not reentrant and
  will cause unexpected happenings if used inside an interrupt.

  Before exiting the function passes the End of Interrupt (EOI) command
  to the 8259 interrupt controller. Don't forget the EOI command; it is
  required even if your ISR does nothing.
****************************************************************************/

#ifdef __cplusplus
 void interrupt NewISR(...)            // C++ style interrupt function
#else
 void interrupt NewISR(void)           // C style interrupt function
#endif
{
  StartConversion6210(BASE_ADDRESS[BOARD]);				// Start the conversion.

  while(IsADConverting6210(BASE_ADDRESS[BOARD]));		// Check if data is ready.
  Value = ReadADData6210(BASE_ADDRESS[BOARD]);			// Read data.
  ClearIRQ6210(BASE_ADDRESS[BOARD]);						// Clear IRQ on DM6210.

  EndOfIT(IRQ_CHANNEL);                // VERY IMPORTANT !!!
													// 'End of Interrupt' command.
} //NewISR


/****************************************************************************
  SetProgramScreen()

  The SetProgramScreen() function initializes the screen.
****************************************************************************/

void SetProgramScreen(void)
{
  clrscr();

  // Print footer and header.
  gotoxy( 1, 1); cprintf(TitleString6210());
  gotoxy(50, 1); cprintf("Interrupt Example");
  gotoxy( 1,25); cprintf("Press any key to exit. . .");

  window(1, 3, 80, 23);
  gotoxy(25, 8); cprintf("Channel %d :           Volts", CHANNEL + 1);
} //SetProgramScreen


int main(void)
{
  float ActualRate0;					// Actual clock rate returned from timer 0.

  BASE_ADDRESS[BOARD0] = 768;				// Set base address of board 0.
  BOARD = 0;

  SetProgramScreen();

  InitBoard6210(BASE_ADDRESS[BOARD]);                   // Board initializing.

  ActualRate0 = SetUserClock6210(BASE_ADDRESS[BOARD], 0, 8000000.0, 20000.0);	// Initialize User Clock0.
  SetUserClock6210(BASE_ADDRESS[BOARD], 1, ActualRate0, RATE);     				// Initialize User Clock1.

  SetChannel6210(BASE_ADDRESS[BOARD], CHANNEL);		// Load Channel Latch.
  Value = 0;     		                     			// Set 'Value'.

  SetIRQ6210(BASE_ADDRESS[BOARD], 2, IRQ_CHANNEL);		// Set Board IRQ Source User TC out 1.
																		// Set IRQ Channel.

  InitHostIT(NewISR,IRQ_CHANNEL);     	  	// Set the Interrupt Service Routine
														// and enable the appropriate IRQ
														// channel.
  ClearIRQ6210(BASE_ADDRESS[BOARD]);		// Clear IRQ on DM6210.

  while ( !kbhit() ) {                 	// Run until any key pressed.
	 gotoxy(37, 8); cprintf( "%6.2f V",(Value-2048)/ADSLOPE);
														// Display Value stored in NewISR
  } //while
  getch();                             	// Get the pressed character.

  SetIRQ6210(BASE_ADDRESS[BOARD], 2, 0);	// Disable board IRQ.
  RestoreHostIT(IRQ_CHANNEL);          	// Restores the IT vector and disable
														// the appropriate IRQ channel.
  DoneTimer6210(BASE_ADDRESS[BOARD]);		// Reset timer.

  window(1,1,80,25);                   	// Reset original screen coordinates,
  clrscr();                            	// and clear the screen.
  return 0;
} //main
